# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'login_dialog_ui.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_UserLoginDialog(object):
    def setupUi(self, UserLoginDialog):
        UserLoginDialog.setObjectName("UserLoginDialog")
        UserLoginDialog.resize(332, 232)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(UserLoginDialog.sizePolicy().hasHeightForWidth())
        UserLoginDialog.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(UserLoginDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame_2 = QtWidgets.QFrame(UserLoginDialog)
        self.frame_2.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.frame_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.frame_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.LoginLE = QtWidgets.QLineEdit(self.frame_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LoginLE.sizePolicy().hasHeightForWidth())
        self.LoginLE.setSizePolicy(sizePolicy)
        self.LoginLE.setMinimumSize(QtCore.QSize(240, 44))
        self.LoginLE.setMaximumSize(QtCore.QSize(240, 16777215))
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setBold(True)
        font.setWeight(75)
        self.LoginLE.setFont(font)
        self.LoginLE.setText("")
        self.LoginLE.setAlignment(QtCore.Qt.AlignCenter)
        self.LoginLE.setClearButtonEnabled(False)
        self.LoginLE.setObjectName("LoginLE")
        self.horizontalLayout_2.addWidget(self.LoginLE)
        self.verticalLayout.addWidget(self.frame_2)
        self.frame = QtWidgets.QFrame(UserLoginDialog)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.frame)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_3 = QtWidgets.QLabel(self.frame)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_3.setFont(font)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.PassLE = QtWidgets.QLineEdit(self.frame)
        self.PassLE.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.PassLE.sizePolicy().hasHeightForWidth())
        self.PassLE.setSizePolicy(sizePolicy)
        self.PassLE.setMinimumSize(QtCore.QSize(240, 44))
        self.PassLE.setMaximumSize(QtCore.QSize(240, 16777215))
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setBold(True)
        font.setWeight(75)
        self.PassLE.setFont(font)
        self.PassLE.setAlignment(QtCore.Qt.AlignCenter)
        self.PassLE.setObjectName("PassLE")
        self.horizontalLayout.addWidget(self.PassLE)
        self.verticalLayout.addWidget(self.frame)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.EnterPB = QtWidgets.QPushButton(UserLoginDialog)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.EnterPB.setFont(font)
        self.EnterPB.setObjectName("EnterPB")
        self.verticalLayout.addWidget(self.EnterPB)
        self.ExitPB = QtWidgets.QPushButton(UserLoginDialog)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.ExitPB.setFont(font)
        self.ExitPB.setObjectName("ExitPB")
        self.verticalLayout.addWidget(self.ExitPB)

        self.retranslateUi(UserLoginDialog)
        QtCore.QMetaObject.connectSlotsByName(UserLoginDialog)

    def retranslateUi(self, UserLoginDialog):
        _translate = QtCore.QCoreApplication.translate
        UserLoginDialog.setWindowTitle(_translate("UserLoginDialog", "GB Чат"))
        self.label_2.setText(_translate("UserLoginDialog", "Логин:"))
        self.LoginLE.setPlaceholderText(_translate("UserLoginDialog", "Логин"))
        self.label_3.setText(_translate("UserLoginDialog", "Пароль:"))
        self.PassLE.setPlaceholderText(_translate("UserLoginDialog", "Пароль"))
        self.EnterPB.setText(_translate("UserLoginDialog", "ВОЙТИ"))
        self.ExitPB.setText(_translate("UserLoginDialog", "УЙТИ"))



from sqlalchemy import create_engine, Column, Integer, String, Text, DateTime, ForeignKey
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, scoped_session
from datetime import datetime


class ClientDB:
    Base = declarative_base()

    # Список всех пользователей
    class AllUsers(Base):
        __tablename__ = 'all_users'
        id = Column(Integer, primary_key=True)
        user_name = Column(String, unique=True)

        def __init__(self, user_name):
            self.user_name = user_name

    # Список контактов пользователя
    class UsersContacts(Base):
        __tablename__ = 'my_contacts'
        id = Column(Integer, primary_key=True)
        contact_name = Column(String, unique=True)

        def __init__(self, contact_name):
            self.contact_name = contact_name

    # Список сообщений
    class MessageHistory(Base):
        __tablename__ = 'my_messages'
        id = Column(Integer, primary_key=True)
        contact = Column(String)
        direction = Column(String)
        msg = Column(Text)
        date = Column(DateTime)

        def __init__(self, contact, direction, msg):
            self.contact = contact
            self.direction = direction
            self.msg = msg
            self.date = datetime.now()

    # Инициализация
    def __init__(self, path):  # client/server_base.db3
        self.engine = create_engine(f'sqlite:///{path}?check_same_thread=False',
                                    echo=False, pool_recycle=7200, encoding='utf-8')
        self.Base.metadata.create_all(self.engine)

        Session = scoped_session(sessionmaker(bind=self.engine))
        self.session = Session()

        self.session.query(self.UsersContacts).delete()
        self.session.commit()

    # Добавление пользователя в контакты
    def add_contact(self, contact_name):
        if not self.session.query(self.UsersContacts).filter_by(contact_name=contact_name).count():
            self.session.add(self.UsersContacts(contact_name))
            self.session.commit()

    # Добавление пользователя в контакты
    def del_contact(self, contact_name):
        self.session.query(self.UsersContacts).filter_by(contact_name=contact_name).delete()
        self.session.commit()

    # Пересоздание списка пользователей
    def update_users(self, users_list):
        self.session.query(self.AllUsers).delete()
        for user in users_list:
            self.session.add(self.AllUsers(user))
        self.session.commit()

    # Пересоздание списка пользователей
    def update_contacts(self, contacts_list):
        self.session.query(self.UsersContacts).delete()
        for user in contacts_list:
            self.session.add(self.UsersContacts(user))
        self.session.commit()

    # Добавление сообщения
    def add_message(self, contact, direction, msg):
        self.session.add(self.MessageHistory(contact, direction, msg))
        self.session.commit()

    # Чтение пользователей
    def get_users(self):
        return [user[0] for user in self.session.query(self.AllUsers.user_name).all()]

    # Чтение контактов
    def get_contacts(self):
        return [contact[0] for contact in self.session.query(self.UsersContacts.contact_name).all()]

    # Чтение сообщений
    def get_history(self, contact):
        query = self.session.query(self.MessageHistory).filter_by(contact=contact)
        return [(m.contact, m.direction, m.msg, m.date) for m in query.all()]

    # Чтение сообщений
    # def get_messages(self, contact=None, direction=None):
    #     query = self.session.query(self.MessageHistory)
    #     if contact:
    #         query = query.filter_by(contact=contact)
    #     if direction:
    #         query = query.filter_by(direction=direction)
    #     return [(m.contact, m.direction, m.msg, str(m.date)) for m in query.all()]

    # Функция проверяет наличие пользователя в таблице Известных Пользователей
    def check_user(self, name):
        if self.session.query(self.AllUsers).filter_by(user_name=name).count():
            return True
        else:
            return False

    # Функция проверяет наличие пользователя в таблице Контактов
    def check_contact(self, contact):
        if self.session.query(self.UsersContacts).filter_by(contact_name=contact).count():
            return True
        else:
            return False


# отладка
if __name__ == '__main__':
    test_db = ClientDB('test1')
    for i in ['test3', 'test4', 'test5']:
        test_db.add_contact(i)
    test_db.add_contact('test4')
    test_db.update_users(['test1', 'test2', 'test3', 'test4', 'test5'])
    test_db.add_message('test1', 'test2', f'Привет! я тестовое сообщение #1')
    test_db.add_message('test2', 'test1', f'Привет! я другое тестовое сообщение #2')
    print(test_db.get_contacts())
    print(test_db.get_users())
    print(test_db.get_messages())
    print(test_db.check_user('test1'))
    print(test_db.check_user('test10'))
    print(test_db.get_messages('test2'))
    print(test_db.get_messages(direction='test2'))
    print(test_db.get_messages('test3'))
    test_db.del_contact('test4')
    print(test_db.get_contacts())



import logging

RESPONSE_1 = 1
LOGGING_LEVEL = logging.DEBUG
ENCODING = 'utf-8'
MAX_PACKAGE_LENGTH = 2048
MAX_CONNECTIONS = 5
DEF_IP = '127.0.0.1'
DEF_PORT = 7777

ACTION = 'action'
TIME = 'time'
USER = 'user'
ACCOUNT_NAME = 'account_name'
SENDER = 'from'
DESTINATION = 'to'
DATA = 'bin'
PUBLIC_KEY = 'pubkey'

MESSAGE = 'message'
MESSAGE_TEXT = 'mess_text'
EXIT = 'exit'
PRESENCE = 'presence'
RESPONSE = 'response'
ERROR = 'error'
ALERT = 'alert'
USER_LIST = 'user_list'
ONLINE = 'online'
GET_CONTACTS = 'get_contacts'
ADD_CONTACT = 'add_contact'
DEL_CONTACT = 'del_contact'
DEF_IP_FOR_RESPONSE = 'def_ip_for_response'
PUBLIC_KEY_REQUEST = 'pubkey_need'
LIST_INFO = 'data_list'

# 200
RESPONSE_200 = {RESPONSE: 200}
# 202
RESPONSE_202 = {RESPONSE: 202,
                LIST_INFO: None
                }
# 400
RESPONSE_400 = {
    RESPONSE: 400,
    ERROR: None
}
# 205
RESPONSE_205 = {
    RESPONSE: 205
}

# 511
RESPONSE_511 = {
    RESPONSE: 511,
    DATA: None
}



"""
It is a launcher for starting subprocesses for server_dist and clients of two types: senders and listeners.
for more information:
https://stackoverflow.com/questions/67348716/kill-process-do-not-kill-the-subprocess-and-do-not-close-a-terminal-window
"""

import os
import signal
import subprocess
import sys
from time import sleep


def main(clients):
    PYTHON_PATH = sys.executable
    BASE_PATH = os.path.dirname(os.path.abspath(__file__))

    def get_subprocess(file_with_args):
        sleep(0.4)
        file_full_path = f"{PYTHON_PATH} {BASE_PATH}/{file_with_args}"
        args = ["gnome-terminal", "--disable-factory", "--", "bash", "-c", file_full_path]
        return subprocess.Popen(args, preexec_fn=os.setpgrp)

    process = []
    while True:
        TEXT_FOR_INPUT = "Выберите действие: q - выход, s - запустить сервер и клиенты, x - закрыть все окна: "
        action = input(TEXT_FOR_INPUT)

        if action == "q":
            break
        elif action == "s":
            process.append(get_subprocess("start_server.py"))

            for i in range(clients):
                process.append(get_subprocess(f"start_client.py "
                                              f"-a 127.0.0.1 "
                                              f"-p 7777 "
                                              f"-name test{i + 1} "
                                              f"-pwd 123456"))

        elif action == "x":
            while process:
                victim = process.pop()
                os.killpg(victim.pid, signal.SIGINT)


if __name__ == '__main__':
    main(3)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        MainWindow.setMinimumSize(QtCore.QSize(800, 600))
        MainWindow.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.ContactFrame = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ContactFrame.sizePolicy().hasHeightForWidth())
        self.ContactFrame.setSizePolicy(sizePolicy)
        self.ContactFrame.setMinimumSize(QtCore.QSize(200, 0))
        self.ContactFrame.setMaximumSize(QtCore.QSize(300, 16777215))
        self.ContactFrame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.ContactFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.ContactFrame.setObjectName("ContactFrame")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.ContactFrame)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.usersTab = QtWidgets.QTabWidget(self.ContactFrame)
        self.usersTab.setEnabled(True)
        self.usersTab.setMaximumSize(QtCore.QSize(300, 16777215))
        font = QtGui.QFont()
        font.setPointSize(14)
        self.usersTab.setFont(font)
        self.usersTab.setElideMode(QtCore.Qt.ElideRight)
        self.usersTab.setObjectName("usersTab")
        self.Conts = QtWidgets.QWidget()
        self.Conts.setObjectName("Conts")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.Conts)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.ContactList = QtWidgets.QListView(self.Conts)
        self.ContactList.setObjectName("ContactList")
        self.verticalLayout_4.addWidget(self.ContactList)
        self.UpdateContsBT = QtWidgets.QPushButton(self.Conts)
        self.UpdateContsBT.setObjectName("UpdateContsBT")
        self.verticalLayout_4.addWidget(self.UpdateContsBT)
        self.usersTab.addTab(self.Conts, "")
        self.Users = QtWidgets.QWidget()
        self.Users.setObjectName("Users")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.Users)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.UserList = QtWidgets.QListView(self.Users)
        self.UserList.setObjectName("UserList")
        self.verticalLayout_2.addWidget(self.UserList)
        self.UpdateUsersBT = QtWidgets.QPushButton(self.Users)
        self.UpdateUsersBT.setObjectName("UpdateUsersBT")
        self.verticalLayout_2.addWidget(self.UpdateUsersBT)
        self.usersTab.addTab(self.Users, "")
        self.verticalLayout_3.addWidget(self.usersTab)
        self.horizontalLayout_2.addWidget(self.ContactFrame)
        self.MsgFrame = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.MsgFrame.sizePolicy().hasHeightForWidth())
        self.MsgFrame.setSizePolicy(sizePolicy)
        self.MsgFrame.setMinimumSize(QtCore.QSize(200, 400))
        self.MsgFrame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.MsgFrame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.MsgFrame.setObjectName("MsgFrame")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.MsgFrame)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame_2 = QtWidgets.QFrame(self.MsgFrame)
        self.frame_2.setMinimumSize(QtCore.QSize(0, 30))
        self.frame_2.setMaximumSize(QtCore.QSize(16777215, 30))
        font = QtGui.QFont()
        font.setPointSize(14)
        self.frame_2.setFont(font)
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.frame_2)
        self.horizontalLayout.setContentsMargins(4, 0, 4, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.ContactStatus = QtWidgets.QLabel(self.frame_2)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(False)
        font.setWeight(50)
        self.ContactStatus.setFont(font)
        self.ContactStatus.setObjectName("ContactStatus")
        self.horizontalLayout.addWidget(self.ContactStatus)
        spacerItem = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.ContactName = QtWidgets.QLabel(self.frame_2)
        self.ContactName.setObjectName("ContactName")
        self.horizontalLayout.addWidget(self.ContactName)
        spacerItem1 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.SettingsBT = QtWidgets.QPushButton(self.frame_2)
        self.SettingsBT.setObjectName("SettingsBT")
        self.horizontalLayout.addWidget(self.SettingsBT)
        self.verticalLayout.addWidget(self.frame_2)
        self.frame = QtWidgets.QFrame(self.MsgFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.MsgField = QtWidgets.QListView(self.frame)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.MsgField.setFont(font)
        self.MsgField.setObjectName("MsgField")
        self.verticalLayout_5.addWidget(self.MsgField)
        self.frame_3 = QtWidgets.QFrame(self.frame)
        self.frame_3.setMinimumSize(QtCore.QSize(0, 40))
        self.frame_3.setMaximumSize(QtCore.QSize(16777215, 70))
        self.frame_3.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_3.setObjectName("frame_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.frame_3)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.EnterTE = QtWidgets.QTextEdit(self.frame_3)
        font = QtGui.QFont()
        font.setFamily("Segoe UI")
        font.setPointSize(12)
        self.EnterTE.setFont(font)
        self.EnterTE.setObjectName("EnterTE")
        self.horizontalLayout_3.addWidget(self.EnterTE)
        self.EnterBT = QtWidgets.QPushButton(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.EnterBT.sizePolicy().hasHeightForWidth())
        self.EnterBT.setSizePolicy(sizePolicy)
        self.EnterBT.setMinimumSize(QtCore.QSize(40, 40))
        font = QtGui.QFont()
        font.setPointSize(14)
        self.EnterBT.setFont(font)
        self.EnterBT.setObjectName("EnterBT")
        self.horizontalLayout_3.addWidget(self.EnterBT)
        self.verticalLayout_5.addWidget(self.frame_3)
        self.verticalLayout.addWidget(self.frame)
        self.horizontalLayout_2.addWidget(self.MsgFrame)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.usersTab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "GB Чат 3000"))
        self.UpdateContsBT.setText(_translate("MainWindow", "Обновить"))
        self.usersTab.setTabText(self.usersTab.indexOf(self.Conts), _translate("MainWindow", "Контакты"))
        self.UpdateUsersBT.setText(_translate("MainWindow", "Обновить"))
        self.usersTab.setTabText(self.usersTab.indexOf(self.Users), _translate("MainWindow", "Добавить"))
        self.ContactStatus.setText(_translate("MainWindow", "Онлайн: "))
        self.ContactName.setText(_translate("MainWindow", "Вася"))
        self.SettingsBT.setText(_translate("MainWindow", "Настройки"))
        self.EnterBT.setText(_translate("MainWindow", "Ввод"))



"""
1. Реализовать графический интерфейс пользователя на стороне клиента:
        a. Отображение списка контактов;
        b. Выбор чата двойным кликом на элементе списка контактов;
        c. Добавление нового контакта в локальный список контактов;
        d. Отображение сообщений в окне чата;
        e. Набор сообщения в окне ввода сообщения;
        f. Отправка введенного сообщения.
"""

# CLIENT
import json
import sys
import threading
import hashlib
import hmac
import binascii
import logging
import log.client_log_config
from PyQt5.QtCore import pyqtSignal, QObject
from time import time, sleep
from socket import socket, AF_INET, SOCK_STREAM

from common.globals import *
from common.utils import send_message, get_message
from common.errors import ReqFieldMissingError, ServerError
from log.decorator import log

LOGGER = logging.getLogger('client')

# Объекты блокировки сокета и работы с базой данных
sock_lock = threading.Lock()
database_lock = threading.Lock()


# Метакласс
# class ClientVerifier(type):
#     def __init__(cls, name, parent, attrs):
#         attrs_required = ['get_message', 'send_message', 'accept', 'listen', 'socket']
#         attrs_not_allowed = []
#         required_present = False
#         for attr in attrs:
#             try:
#                 decompiled = dis.get_instructions(attrs[attr])
#             except TypeError:
#                 pass
#             else:
#                 for i in decompiled:
#                     if i.opname == 'LOAD_GLOBAL':
#                         if i.argval in attrs_required:
#                             required_present = True
#                         if i.argval in attrs_not_allowed:
#                             raise Exception(f'В классе {name} используется недопустимое: {i.argval}')
#         if not required_present:
#             raise Exception(f'В классе {name} отсутствует необходимое: {attrs_required}')
#         super().__init__(name, parent, attrs)


class ClientTransport(threading.Thread, QObject):
    """Класс обработки сообщений и работы сокета. Принимает:
    базу данных, имя, пароль, ip-адрес сервера, порт сервера, ключ
    """
    # Сигналы новое сообщениеб отобразить сообщение и потеря соединения
    new_message = pyqtSignal(str)
    message_205 = pyqtSignal()
    connection_lost = pyqtSignal()
    show_message = pyqtSignal(str)

    def __init__(self, database, acc_name, acc_pwd, serv_ip, serv_port, keys):  # database, acc_name, serv_ip, serv_port
        # Вызываем конструктор предка
        threading.Thread.__init__(self)
        QObject.__init__(self)

        # Класс База данных - работа с базой
        self.database = database
        # Имя пользователя
        self.acc_name = acc_name
        # Пароль
        self.acc_pwd = acc_pwd
        # Сокет для работы с сервером
        self.transport = None
        # Набор ключей для шифрования
        self.keys = keys
        # Устанавливаем соединение:
        self.connection_init(serv_ip, serv_port)
        # Обновляем таблицы известных пользователей и контактов
        try:
            self.update_db_list(USER_LIST)
            self.update_db_list(GET_CONTACTS)
        except OSError as err:
            if err.errno:
                LOGGER.critical(f'Потеряно соединение с сервером.')
                raise ServerError('Потеряно соединение с сервером!')
            LOGGER.error('Timeout соединения при обновлении списков пользователей.')
        except json.JSONDecodeError:
            LOGGER.critical(f'Потеряно соединение с сервером.')
            raise ServerError('Потеряно соединение с сервером!')
        # Флаг продолжения работы транспорта.
        self.running = True

    def connection_init(self, serv_ip, serv_port):
        """Метод инициализатор сокета. Принимает:
        ip-адрес сервера, порт сервера
        """
        try:
            self.transport = socket(AF_INET, SOCK_STREAM)
            self.transport.settimeout(5)

            connected = False
            for knock in range(5):
                # Если имя подходит, создаем подключение
                try:
                    LOGGER.debug(f'[{self.acc_name}]: Попытка соедиения...')
                    self.transport.connect((serv_ip, serv_port))
                    LOGGER.debug(f'Соединение установлено.')
                    connected = True
                    break
                except (ConnectionRefusedError, ConnectionError, OSError):
                    LOGGER.debug(f'Сервер не отвечает {serv_ip}:{serv_port}')
                sleep(1)

            # Если не удалось ввести имя или подключиться 5 раз - расходимся
            if not connected:
                LOGGER.debug(f'Сервер не отвечает {serv_ip}:{serv_port}. Завершение работы.')
                sys.exit(0)

        except json.JSONDecodeError:
            LOGGER.error('Не удалось декодировать полученную Json строку.')
            sys.exit(1)
        except ServerError as error:
            LOGGER.error(f'При установке соединения сервер вернул ошибку: {error.text}')
            sys.exit(1)
        except ReqFieldMissingError as missing_error:
            LOGGER.error(f'В ответе сервера отсутствует необходимое поле {missing_error.missing_field}')
            sys.exit(1)
        else:
            LOGGER.info(f'Клиент запущен. Использую SERVER IP:{serv_ip} PORT:{serv_port} NAME:{self.acc_name}')
            # Запускаем процедуру авторизации
            # Получаем хэш пароля
            passwd_bytes = self.acc_pwd.encode('utf-8')
            salt = self.acc_name.lower().encode('utf-8')
            passwd_hash = hashlib.pbkdf2_hmac('sha512', passwd_bytes, salt, 10000)
            passwd_hash_string = binascii.hexlify(passwd_hash)

            LOGGER.debug(f'Passwd hash ready: {passwd_hash_string}')

            # Получаем публичный ключ и декодируем его из байтов
            pubkey = self.keys.publickey().export_key().decode('ascii')

            # Авторизируемся на сервере

            # if self.create_presence_message(self.transport):
            # LOGGER.debug(f'Сервер успешно уведомлен о присутствии')
            with sock_lock:
                presense = {
                    ACTION: PRESENCE,
                    TIME: time(),
                    USER: {
                        ACCOUNT_NAME: self.acc_name,
                        PUBLIC_KEY: pubkey
                    }
                }
                LOGGER.debug(f"Presense message = {presense}")
                # Отправляем серверу приветственное сообщение.
                try:
                    send_message(self.transport, presense)
                    ans = get_message(self.transport)
                    LOGGER.debug(f'Server response = {ans}.')
                    # Если сервер вернул ошибку, бросаем исключение.
                    if RESPONSE in ans:
                        if ans[RESPONSE] == 400:
                            self.show_message.emit(ans[ERROR])
                            raise ServerError(ans[ERROR])
                        elif ans[RESPONSE] == 511:
                            # Если всё нормально, то продолжаем процедуру
                            # авторизации.
                            ans_data = ans[DATA]
                            hash = hmac.new(passwd_hash_string, ans_data.encode('utf-8'), 'MD5')
                            digest = hash.digest()
                            my_ans = RESPONSE_511
                            my_ans[DATA] = binascii.b2a_base64(
                                digest).decode('ascii')
                            send_message(self.transport, my_ans)
                            self.process_server_ans(get_message(self.transport))
                except (OSError, json.JSONDecodeError) as err:
                    LOGGER.debug(f'Connection error.', exc_info=err)
                    raise ServerError('Сбой соединения в процессе авторизации.')

    def run(self):
        """Метод основного цикла, получает и распределяет сообщения"""
        while self.running:
            sleep(1)
            with sock_lock:
                try:
                    self.transport.settimeout(0.5)
                    msg = get_message(self.transport)
                except OSError as err:
                    if err.errno:
                        LOGGER.error(f'Потеряно соединение с сервером (таймаут)')
                        self.running = False
                        self.connection_lost.emit()
                except (ConnectionError, ConnectionAbortedError,
                        ConnectionResetError, json.JSONDecodeError, TypeError) as err:
                    LOGGER.error(f'Потеряно соединение с сервером: {err}')
                    self.running = False
                    self.connection_lost.emit()
                else:
                    if msg:
                        #     LOGGER.debug(f'Соединение с сервером разорвано.')
                        if ACTION in msg:
                            if msg[ACTION] == MESSAGE and TIME in msg and SENDER in msg and MESSAGE_TEXT in msg:
                                # if msg[SENDER] != self.acc_name:
                                LOGGER.debug(f'Получено сообщение от {msg[SENDER]}: {msg[MESSAGE_TEXT]}')
                                with database_lock:
                                    try:
                                        self.database.add_message(msg[SENDER], 'i', msg[MESSAGE_TEXT])
                                        self.new_message.emit(msg[SENDER])
                                    except Exception as err:
                                        LOGGER.error(f'Ошибка взаимодействия с базой данных: {err}')
                                # if DESTINATION in msg and msg[DESTINATION] == self.acc_name:
                                #     print(f'ЛИЧНО [{msg[SENDER]}]: {msg[MESSAGE_TEXT]}')
                                # else:
                                #     print(f'[{msg[SENDER]}]: {msg[MESSAGE_TEXT]}')
                                # LOGGER.debug(f'Получено сообщение от пользователя {msg[SENDER]}: {msg[MESSAGE_TEXT]}')
                        elif RESPONSE in msg:
                            self.process_server_ans(msg)
                        else:
                            LOGGER.error(f'Получено некорректное сообщение с сервера: {msg}')
                finally:
                    self.transport.settimeout(5)

    # Exit
    def exit(self):
        """Метод выхода из программы"""
        self.send_service_message(EXIT)
        LOGGER.info('Завершение работы по команде пользователя.')
        sleep(0.3)

    # Добавляем контакт
    def add_contact(self, contact):
        """Метод добавляет контакт в список контактов. Принимает:
        имя
        """
        # LOGGER.debug(f'Отправляю команду "{ADD_CONTACT} {contact}"')
        if self.send_service_message(ADD_CONTACT, contact):
            with database_lock:
                try:
                    self.database.add_contact(contact)
                except Exception as err:
                    LOGGER.error(f'Контакт "{contact}" добавить не удалось: {err}')
                else:
                    return True
        else:
            LOGGER.error(f'Контакт "{contact}" добавить не удалось, ошибка сервера')

    # Удаляем контакт
    def del_contact(self, contact):
        """Метод удаляет контакт из списка контактов. Принимает:
        имя
        """
        # LOGGER.debug(f'Отправляю команду "{DEL_CONTACT} {contact}"')
        if self.send_service_message(DEL_CONTACT, contact):
            with database_lock:
                try:
                    self.database.del_contact(contact)
                except Exception as err:
                    LOGGER.error(f'Контакт "{contact}" удалить не удалось: {err}')
                else:
                    return True
        else:
            LOGGER.error(f'Контакт "{contact}" удалить не удалось, ошибка сервера')

    # Функция обновления таблицы известных пользователей.
    def update_db_list(self, command):
        """Метод загружает актуальною базу с сервера и сохраняет локально. Принимает:
         USER_LIST
         GET_CONTACTS
         """
        LOGGER.debug(f'Запрос списка {command}')
        users_list = self.send_service_message(command)

        if users_list:
            with database_lock:
                if command == USER_LIST:
                    self.database.update_users(users_list)
                elif command == GET_CONTACTS:
                    self.database.update_contacts(users_list)
        # else:
        #     LOGGER.error(f'Не удалось {command}, ответ сервера: EMPTY')

    # Запрашиваем список пользователей из базы
    def get_users_list(self):
        """Метод загружает в память локальную базу всех пользователей"""
        # contact_list = self.send_service_message(self.acc_name, USER_LIST)
        with database_lock:
            try:
                user_list = self.database.get_users()
            except Exception as err:
                LOGGER.error(f'Список пользователей загрузить не удалось: {err}')
            else:
                for user in sorted(user_list):
                    print(f'[{user[0]}]')

    # Запрашиваем список контактов из базы
    def get_contact_list(self):
        """Метод загружает в память локальную базу всех контактов текущего пользователя"""
        # contact_list = self.send_service_message(self.acc_name, GET_CONTACTS)
        with database_lock:
            try:
                contact_list = self.database.get_contacts()
            except Exception as err:
                LOGGER.error(f'Контакты загрузить не удалось: {err}')
            else:
                print(f'Ваш список контактов: {contact_list}')

    # Запрашиваем список сообщений
    # def get_msg_history(self):
    #     name = input('Введите "от <имя>" или "к <имя>" пользователя или нажмите Enter: ')
    #     with database_lock:
    #         name_from, name_to = None, None
    #         if name:
    #             if name[0] == 'о':
    #                 name_from = name[2:].strip()
    #             elif name[0] == 'к':
    #                 name_to = name[1:].strip()
    #         try:
    #             msg_list = self.database.get_messages(from_name=name_from, to_name=name_to)
    #         except Exception as err:
    #             LOGGER.error(f'Список пользователей загрузить не удалось: {err}')
    #         else:
    #             for m in sorted(msg_list):
    #                 print(f'({m[3]}) [{m[0]}] => [{m[1]}]: {m[2]}')

    # Запрашиваем список онлайн пользователей
    # def get_online_list(self):
    #     contact_list = self.send_service_message(self.acc_name, ONLINE)
    #     print(f'Пользователи онлайн: {contact_list}')

    # Запрашиваем помощь
    # elif msg[1] in ('h', '?'):
    #     self.print_help()

    # # Пишем личное сообщение
    # def send_message(self, msg):
    #     dest_name, text = self.get_name_from_msg(msg, 1, get_text=True)
    #     if dest_name:
    #         self.send_text_message(self.sock, dest_name, text)
    #     else:
    #         print('Для отправки личного сообщения введите "! <имя> <сообщение>"')
    #
    # # Пишем сообщение всем
    # def send_message_to_all(self, msg):
    #     LOGGER.debug(f'Подгатавливаю сообщение {msg}')
    #     self.send_text_message(self.sock, '', msg)
    #
    #     def get_name_from_msg(self, msg: str, cmd_len: int, get_text=False):
    #         if len(msg) < cmd_len + 1:
    #             return None, None if get_text else None
    #         name_len = msg[cmd_len:].lstrip().find(' ') if get_text else len(msg[cmd_len:].strip())
    #         if name_len <= 0:
    #             return None, None if get_text else None
    #         # print(f'name={msg[cmd_len + 1:cmd_len + 1 + name_len]=}')
    #         # print(f'text={msg[cmd_len + 1 + name_len:]=}')
    #         if get_text:
    #             return msg[cmd_len + 1:cmd_len + 1 + name_len], msg[cmd_len + 2 + name_len:]
    #         return msg[cmd_len + 1:cmd_len + 1 + name_len]

    @log
    def send_text_message(self, dest_name, msg):
        """Метод отправляет серверу текстовое сообщение для получателя. Принимает:
        имя, сообщение
        """
        message_dict = {
            ACTION: MESSAGE,
            TIME: time(),
            SENDER: self.acc_name,
            DESTINATION: dest_name,
            MESSAGE_TEXT: msg
        }
        LOGGER.debug(f'Добавляю сообщение в базу {msg}')
        with database_lock:
            self.database.add_message(dest_name, 'o', msg)

        LOGGER.debug(f'Отправляю сообщение {msg}')
        with sock_lock:
            try:
                send_message(self.transport, message_dict)
                LOGGER.debug(f'Отправлено сообщение {msg} для пользователя {dest_name}')
            except OSError as err:
                if err.errno:
                    LOGGER.critical('Потеряно соединение с сервером.')
                    sys.exit(1)
                else:
                    LOGGER.error('Не удалось передать сообщение. Таймаут соединения')

    # @log
    def send_service_message(self, msg_type, data=None):
        """Метод отправляет сервисное сообщение серверу. Принимаются команды:
        USER_LIST
        ONLINE
        GET_CONTACTS
        EXIT
        а так же:
        ADD_CONTACT, имя
        DEL_CONTACT, имя
        """
        # Формирование простых сообщений USER_LIST, ONLINE, GET_CONTACTS, EXIT
        self.transport.settimeout(1)
        if msg_type in (USER_LIST, ONLINE, GET_CONTACTS, EXIT):
            LOGGER.debug(f'Сформировано "{msg_type}" сообщение для сервера')
            msg = {
                ACTION: msg_type,
                TIME: time(),
                USER: self.acc_name
            }

            with sock_lock:
                try:
                    send_message(self.transport, msg)
                    if msg_type == EXIT:
                        return
                    ans = get_message(self.transport)
                except OSError as err:
                    if err.errno:
                        print(f'Не удалось доставить сообщение: {err}')
                        LOGGER.error(f'Не удалось доставить сообщение: {err}')
                        return
                else:
                    if ans and RESPONSE in ans:
                        LOGGER.debug(f'Получен ответ "{ans[RESPONSE]}" сообщение от сервера')
                        # Ответ OK
                        if ans[RESPONSE] == 200:
                            pass
                        # Ответ OK со списком
                        elif ans[RESPONSE] == 202 and ALERT in ans:
                            return ans[ALERT]
                        else:
                            self.show_message.emit(ans[ERROR])
                            LOGGER.error(f'Не удалось {msg_type}, ответ сервера: {ans}')

        # Формирование сообщений для добавления/удаления контакта DD_CONTACT, DEL_CONTACT
        if data and msg_type in (ADD_CONTACT, DEL_CONTACT):
            LOGGER.debug(f'Сформировано "{msg_type} {data}" сообщение для сервера')
            msg = {
                ACTION: msg_type,
                TIME: time(),
                USER: self.acc_name,
                ACCOUNT_NAME: data
            }
            with sock_lock:
                try:
                    send_message(self.transport, msg)
                    ans = get_message(self.transport)
                except OSError as err:
                    if err.errno:
                        print(f'Не удалось доставить сообщение: {err}')
                        LOGGER.error(f'Не удалось доставить сообщение: {err}')
                        return
                else:
                    if ans and RESPONSE in ans:
                        LOGGER.debug(f'Получен ответ "{ans[RESPONSE]}" сообщение от сервера')
                        if ans[RESPONSE] == 200:
                            LOGGER.debug(f'Получен ответ сервера: "OK"')
                            return True
                        else:
                            self.show_message.emit(ans[ERROR])
                            LOGGER.debug(f'Не удалось {msg_type}, ответ сервера: {ans}')
                            return False
                    LOGGER.debug(f'Получен неожиданный ответ сервера: "{ans}"')
        self.transport.settimeout(5)

    # def create_presence_message(self, sock):
    #     # Формирование простых сообщения PRESENCE
    #     LOGGER.debug(f'Сформировано "PRESENCE" сообщение для сервера')
    #     msg = {
    #         ACTION: PRESENCE,
    #         TIME: time(),
    #         USER: {
    #             ACCOUNT_NAME: self.acc_name
    #         }
    #     }
    #     send_message(sock, msg)
    #     ans = get_message(sock)
    #     if RESPONSE in ans and ans[RESPONSE] == 200:
    #         LOGGER.debug(f'Ответ сервера: "OK"')
    #         return True
    #     self.show_message.emit(ans)
    #     LOGGER.error(f'Ответ сервера: {ans}')

    def process_server_ans(self, msg):
        """Метод обработчик неожиданно поступающих сообщений с сервера."""
        LOGGER.debug(f'Разбор сообщения от сервера: {msg}')

        # Если это подтверждение чего-либо
        if RESPONSE in msg:
            if msg[RESPONSE] == 200:
                return
            elif msg[RESPONSE] == 400:
                self.show_message.emit(msg[ERROR])
                # raise ServerError(f'{msg[ERROR]}')
            elif msg[RESPONSE] == 205:
                self.update_db_list(USER_LIST)
                self.update_db_list(GET_CONTACTS)
                self.message_205.emit()
            else:
                LOGGER.error(
                    f'Принят неизвестный код подтверждения {msg[RESPONSE]}')

    # def print_help(self):
    #     print(' Поддерживаемые команды:')
    #     print('   <сообщение> - отправить сообщение всем')
    #     print('   ! <имя> <сообщение> - отправить личное сообщение')
    #     print('   /+ <имя> - добавить в список контактов')
    #     print('   /- <имя> - удалить из контактов')
    #     print('   /! - запросить список собеседников')
    #     print('   /a или /all - запросить список всех пользователей')
    #     print('   /c или /cont - запросить список контактов')
    #     print('   /m или /msg - запросить историю сообщений')
    #     print('   /h или /? - вывести подсказки по командам')
    #     print('   /q или /й - выход из программы')


# @log
# def create_presence_message(sock, acc_name):
#     # Формирование простых сообщения PRESENCE
#     LOGGER.debug(f'Сформировано "PRESENCE" сообщение для сервера')
#     msg = {
#         ACTION: PRESENCE,
#         TIME: time(),
#         USER: {
#             ACCOUNT_NAME: acc_name
#         }
#     }
#     send_message(sock, msg)
#     ans = get_message(sock)
#     if RESPONSE in ans and ans[RESPONSE] == 200:
#         LOGGER.debug(f'Ответ сервера: "OK"')
#         return True
#     LOGGER.error(f'Ответ сервера: {ans}')
#
#
# def load_database(user_name, database, sock):
#     sender = ClientSender(user_name, database, sock)
#     try:
#         users_list = sender.send_service_message(user_name, USER_LIST)
#     except Exception as err:
#         LOGGER.error(f'Ошибка запроса списка пользователей: {err}')
#     else:
#         if users_list:
#             database.update_users(users_list)
#     try:
#         contacts_list = sender.send_service_message(user_name, GET_CONTACTS)
#     except Exception as err:
#         LOGGER.error(f'Ошибка запроса списка контактов: {err}')
#     else:
#         if contacts_list:
#             for contact in contacts_list:
#                 database.add_contact(contact)
#     LOGGER.debug(f'База успешно загружена с сервера')




import sys
import logging
import inspect


def log(func):
    LOGGER = logging.getLogger('server') if 'server.py' in sys.argv[0] else logging.getLogger('client')

    def decor(*args, **kwargs):
        result = func(*args, **kwargs)
        s = '\\'
        LOGGER.debug(f'"{inspect.getmodule(inspect.stack()[1][0]).__file__.rpartition(s)[-1]}/'
                     f'{inspect.getouterframes(inspect.currentframe())[1][3]}" '
                     f'Вызывает: "{func.__module__}/{func.__name__}("{args}" {kwargs})"; '
                     f'Результат: "{str(result)}"')
        return result

    decor.log = LOGGER
    return decor



import sys

from PyQt5.QtWidgets import QApplication, qApp, QDialog

sys.path.append('../')

from client.login_dialog_ui import Ui_UserLoginDialog


class LoginWindow(QDialog):
    def __init__(self):
        super().__init__()

        self.enter = False

        self.login_window = Ui_UserLoginDialog()
        self.login_window.setupUi(self)

        self.login_window.EnterPB.clicked.connect(self.run)
        self.login_window.ExitPB.clicked.connect(qApp.exit)

        self.show()

    def run(self):
        if self.login_window.LoginLE.text() and self.login_window.PassLE.text():
            self.enter = True
            qApp.exit()


if __name__ == '__main__':
    app = QApplication([])
    dial = LoginWindow()
    app.exec_()



"""
1. Реализовать графический интерфейс пользователя на стороне клиента:
        a. Отображение списка контактов;
        b. Выбор чата двойным кликом на элементе списка контактов;
        c. Добавление нового контакта в локальный список контактов;
        d. Отображение сообщений в окне чата;
        e. Набор сообщения в окне ввода сообщения;
        f. Отправка введенного сообщения.
"""

# CLIENT
import os
import sys
import threading

from Cryptodome.PublicKey import RSA
from PyQt5.QtWidgets import QApplication
from time import sleep, strftime, localtime
# from socket import socket, AF_INET, SOCK_STREAM
from common.globals import *
from common.utils import handle_parameters
# from common.errors import ReqFieldMissingError, ServerError
import logging
from client.client_db import ClientDB
from client.login_dialog import LoginWindow
from client.main_window import ClientMainWindow
from client.transport import ClientTransport
# from client.main_window_ui import Ui_MainWindow

LOGGER = logging.getLogger('client')

# Объект блокировки сокета и работы с базой данных
sock_lock = threading.Lock()
database_lock = threading.Lock()


# @log
# def create_presence_message(sock, acc_name):
#     # Формирование простых сообщения PRESENCE
#     LOGGER.debug(f'Сформировано "PRESENCE" сообщение для сервера')
#     msg = {
#         ACTION: PRESENCE,
#         TIME: time(),
#         USER: {
#             ACCOUNT_NAME: acc_name
#         }
#     }
#     send_message(sock, msg)
#     ans = get_message(sock)
#     if RESPONSE in ans and ans[RESPONSE] == 200:
#         LOGGER.debug(f'Ответ сервера: "OK"')
#         return True
#     LOGGER.error(f'Ответ сервера: {ans}')
#

# def load_database(user_name, database, sock):
#     sender = ClientSender(user_name, database, sock)
#     try:
#         users_list = sender.create_service_message(user_name, USER_LIST)
#     except Exception as err:
#         LOGGER.error(f'Ошибка запроса списка пользователей: {err}')
#     else:
#         if users_list:
#             database.update_users(users_list)
#     try:
#         contacts_list = sender.create_service_message(user_name, GET_CONTACTS)
#     except Exception as err:
#         LOGGER.error(f'Ошибка запроса списка контактов: {err}')
#     else:
#         if contacts_list:
#             for contact in contacts_list:
#                 database.add_contact(contact)
#     LOGGER.debug(f'База успешно загружена с сервера')


if __name__ == '__main__':
    LOGGER.debug('=' * 40 + '[ CLIENT LOG START TIME: ' + strftime("%a, %d %b %Y %H:%M:%S ]", localtime()) + '=' * 40)
    try:
        # Загружаем параметы коммандной строки
        serv_ip, serv_port, acc_name, acc_pwd = handle_parameters(ip=DEF_IP, port=DEF_PORT)

        # Создаём клиентокое приложение
        client_app = QApplication(sys.argv)

        knock = 0
        for knock in range(5):

            # Требуем имя
            if not acc_name or not acc_pwd:
                login_dialog = LoginWindow()
                # login_dialog.setupUi(Ui_UserLoginDialog)
                client_app.exec_()
                # Если пользователь ввёл имя и нажал ОК, то сохраняем ведённое и удаляем объект.
                # Иначе - выходим
                if login_dialog.enter:
                    acc_name = login_dialog.login_window.LoginLE.text()
                    acc_pwd = login_dialog.login_window.PassLE.text()
                    del login_dialog
                else:
                    exit(0)

        # # Если не удалось ввести имя или подключиться 5 раз - расходимся
        # if knock == 4:
        #     print('К сожалению подключиться не удалось. Попробуйте позже.')
        #     LOGGER.debug(f'Сервер не отвечает {serv_ip}:{serv_port}. Завершение работы.')
        #     sys.exit(0)
        # LOGGER.info(f'Клиент запущен. Использую SERVER IP:{serv_ip} PORT:{serv_port} NAME:{acc_name}')

        # Инициализация БД
        db_name = f'{acc_name}.db3'
        dir_path = os.path.dirname(os.path.realpath(__file__)) + '\\client\\'
        key_file = os.path.join(dir_path, f'{acc_name}.key')

        if not os.path.exists(key_file):
            keys = RSA.generate(2048, os.urandom)
            with open(key_file, 'wb') as key:
                key.write(keys.export_key())
        else:
            with open(key_file, 'rb') as key:
                keys = RSA.import_key(key.read())

        try:
            database = ClientDB(dir_path+db_name)
        except Exception as e:
            LOGGER.critical(f'Не удается найти/создать базу "{dir_path+db_name}" Причина: {e}')
        # load_database(acc_name, database, my_sock)

        try:
            transport = ClientTransport(database, acc_name, acc_pwd, serv_ip, serv_port, keys)
        except Exception as err:
            LOGGER.error(f'Ошибка при запуске клиента: {err}')
            exit(1)
        transport.setDaemon(True)
        transport.start()
        sleep(0.2)

        # Создаём GUI
        main_window = ClientMainWindow(database, transport)
        main_window.make_connection(transport)
        main_window.setWindowTitle(f'Чат Программа alpha release - {acc_name}')
        client_app.exec_()

        # Раз графическая оболочка закрылась, закрываем транспорт
        transport.exit()
        transport.join()

        # thread_receiver = ClientReader(acc_name, database, my_sock)
        # thread_receiver.daemon = True
        # thread_receiver.start()
        # LOGGER.debug('Запущен процесс получатель')
        #
        # thread_sender_ui = ClientSender(acc_name, database, my_sock)
        # thread_sender_ui.daemon = True
        # thread_sender_ui.start()
        # LOGGER.debug('Запущен процесс интерфейс и отправщик')
        #
        # while thread_receiver.is_alive() and thread_sender_ui.is_alive():
        #     sleep(1)

    except KeyboardInterrupt:
        LOGGER.info(f'Завершение работы клиента')
        # my_sock.close()
        LOGGER.info(f'Клиент остановлен')
        sys.exit(1)



from PyQt5.QtWidgets import QMainWindow, QMessageBox, QApplication, QMenu
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor
from PyQt5.QtCore import pyqtSlot, QEvent, Qt
import sys

sys.path.append('../')
from common.globals import *
from client.main_window_ui import Ui_MainWindow

LOGGER = logging.getLogger('client')


class ClientMainWindow(QMainWindow):
    def __init__(self, database, transport):
        super().__init__()

        # основные переменные
        self.database = database
        self.transport = transport

        # Загружаем конфигурацию окна из дизайнера
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        # Кнопка "Выход"
        # self.ui.menu_exit.triggered.connect(qApp.exit)

        # Кнопка отправить сообщение
        self.ui.EnterBT.clicked.connect(self.send_message)

        # Панель пользователей и контактов
        # self.ui.usersTab.installEventFilter(self)

        self.ui.UserList.installEventFilter(self)
        self.ui.UpdateUsersBT.clicked.connect(self.update_lists)

        self.ui.ContactList.installEventFilter(self)
        self.ui.UpdateContsBT.clicked.connect(self.update_lists)

        # "добавить контакт"
        # self.ui.add_contact_context = QListWidget()
        # self.ui.add_contact_context.addItem('Добавить контакт')
        # self.ui.UserList.contextMenuEvent.connect(self.add_contact_context)

        # self.ui.UserList.customContextMenuRequested.connect(self.add_contact_context)
        # QObject.connect(self.pushButton, PYQT_SIGNAL("clicked()"), self.add_contact_context)
        # self.ui.btn_add_contact.clicked.connect(self.add_contact_window)
        # self.ui.menu_add_contact.triggered.connect(self.add_contact_window)

        # Удалить контакт

        # self.ui.btn_remove_contact.clicked.connect(self.delete_contact_window)
        # self.ui.menu_del_contact.triggered.connect(self.delete_contact_window)

        # Дополнительные требующиеся атрибуты
        self.contacts_model = None
        self.users_model = None
        self.history_model = None
        self.messages = QMessageBox()
        self.current_chat = None  # Текущий контакт с которым идёт обмен сообщениями
        self.ui.MsgField.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.ui.MsgField.setWordWrap(True)

        # Double click по списку контактов отправляется в обработчик
        self.ui.ContactList.doubleClicked.connect(self.select_active_user)

        self.update_contacts_list()
        self.update_users_list()
        self.set_disabled_input()
        self.show()

    def eventFilter(self, source, event):
        if event.type() == QEvent.ContextMenu:
            # print(source.indexAt(event.pos()))
            if source is self.ui.UserList:
                menu = QMenu()
                menu.addAction('Добавить контакт')
                try:
                    if menu.exec_(event.globalPos()):
                        item = source.indexAt(event.pos())
                        LOGGER.debug(f'Добавляю контакт: {item.data()}')
                        if item and not self.database.check_contact(item.data()):
                            self.ui.usersTab.setCurrentIndex(1)
                            self.transport.add_contact(item.data())
                            self.update_contacts_list()
                    return True
                except Exception as e:
                    LOGGER.error(f'Не удалось добавить контакт: {e}')

            if source is self.ui.ContactList:
                menu = QMenu()
                menu.addAction('Удалить контакт')
                try:
                    if menu.exec_(event.globalPos()):
                        item = source.indexAt(event.pos())
                        LOGGER.debug(f'Удаляю контакт: {item.data()}')
                        if item and self.database.check_contact(item.data()):
                            self.transport.del_contact(item.data())
                            self.update_contacts_list()
                    return True
                except Exception as e:
                    LOGGER.error(f'Не удалось удалить контакт: {e}')
        return super().eventFilter(source, event)

    # Деактивировать поля ввода
    def set_disabled_input(self):
        # Надпись  - получатель.
        self.ui.ContactName.setText('Выберите пользователя')
        self.ui.EnterTE.clear()
        if self.history_model:
            self.history_model.clear()

        # Поле ввода и кнопка отправки неактивны до выбора получателя.
        self.ui.EnterBT.setDisabled(True)
        self.ui.EnterTE.setDisabled(True)

    def update_lists(self):
        if self.ui.usersTab.currentWidget().objectName() == 'Users':
            self.transport.update_db_list(USER_LIST)
            self.update_users_list()
        elif self.ui.usersTab.currentWidget().objectName() == 'Conts':
            self.transport.update_db_list(GET_CONTACTS)
            self.update_contacts_list()
        # print(self.ui.usersTab.currentWidget().objectName())

    # Функция, обновляющая контакт-лист
    def update_contacts_list(self):
        LOGGER.debug('Запрос скиска контактов')
        contacts_list = self.database.get_contacts()
        self.contacts_model = QStandardItemModel()
        for i in sorted(contacts_list):
            item = QStandardItem(i)
            item.setEditable(False)
            self.contacts_model.appendRow(item)
        self.ui.ContactList.setModel(self.contacts_model)

    # Функция, обновляющая лист пользователей
    def update_users_list(self):
        LOGGER.debug('Запрос скиска пользователей')
        users_list = self.database.get_users()
        self.users_model = QStandardItemModel()
        for i in sorted(users_list):
            item = QStandardItem(i)
            item.setEditable(False)
            self.users_model.appendRow(item)
        self.ui.UserList.setModel(self.users_model)

    # Функция обработчик double click по контакту
    def select_active_user(self):
        # Выбранный пользователем контакт находится в выделенном элементе в QListView
        self.current_chat = self.ui.ContactList.currentIndex().data()
        # вызываем основную функцию
        self.set_active_user()

    # Функция, устанавливающая активного собеседника
    def set_active_user(self):
        # Ставим надпись и активируем кнопки
        self.ui.ContactName.setText(self.current_chat)
        self.ui.EnterBT.setDisabled(False)
        self.ui.EnterTE.setDisabled(False)

        # Заполняем окно историю сообщений по требуемому пользователю.
        self.history_list_update()

    # Функция отправки сообщения пользователю.
    def send_message(self):
        # Текст в поле, проверяем что поле не пустое затем забирается сообщение и поле очищается
        msg_text = self.ui.EnterTE.toPlainText()
        self.ui.EnterTE.clear()
        if not msg_text:
            return
        self.transport.send_text_message(self.current_chat, msg_text)
        self.history_list_update()
        # try:
        #     self.transport.send_text_message(self.current_chat, message_text)
        # except ServerError as err:
        #     self.messages.critical(self, 'Ошибка', err.text)
        # except OSError as err:
        #     if err.errno:
        #         self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером!')
        #         self.close()
        #     self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
        # except (ConnectionResetError, ConnectionAbortedError):
        #     self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером!')
        #     self.close()
        # else:
        #     self.database.save_message(self.current_chat, 'o', message_text)
        #     LOGGER.debug(f'Отправлено сообщение для {self.current_chat}: {message_text}')
            # self.history_list_update()

    # Заполняем историю сообщений.
    def history_list_update(self):
        # Получаем историю сортированную по дате
        LOGGER.debug(f'Загрузка истории сообщений')
        list_messages = sorted(self.database.get_history(self.current_chat),
                               key=lambda item: item[3])
        # Если модель не создана, создадим.
        if not self.history_model:
            LOGGER.debug(f'Создание модели истории сообщений')
            self.history_model = QStandardItemModel()
            self.ui.MsgField.setModel(self.history_model)
        # Очистим от старых записей
        self.history_model.clear()
        # Берём не более 20 последних записей.
        # LOGGER.debug(f'Выводим последние 20 сообщений')
        length = len(list_messages)
        start_index = 0
        if length > 20:
            start_index = length - 20
        # Заполнение модели записями, так же стоит разделить входящие и исходящие
        # сообщения выравниванием и разным фоном.
        # Записи в обратном порядке, поэтому выбираем их с конца и не более 20
        for i in range(start_index, length):
            item = list_messages[i]
            if item[1] == 'i':
                mess = QStandardItem(f'{self.current_chat} ({item[3].replace(microsecond=0)}):\n {item[2]}')
                mess.setEditable(False)
                mess.setBackground(QBrush(QColor(224, 233, 255)))
                mess.setTextAlignment(Qt.AlignLeft)
                self.history_model.appendRow(mess)
            else:
                mess = QStandardItem(f'{self.transport.acc_name} ({item[3].replace(microsecond=0)}):\n {item[2]}')
                mess.setEditable(False)
                mess.setTextAlignment(Qt.AlignRight)
                mess.setBackground(QBrush(QColor(224, 255, 233)))
                self.history_model.appendRow(mess)
        self.ui.MsgField.scrollToBottom()

    @pyqtSlot(str)
    def show_message(self, msg):
        if msg:
            self.messages.question(self, 'Внимание', f'SERVER: {msg}, ', QMessageBox.Close)

    # Слот приёма нового сообщений
    @pyqtSlot(str)
    def message(self, sender):
        if sender == self.current_chat:
            self.history_list_update()
        else:
            # Проверим есть ли такой пользователь у нас в контактах:
            if self.database.check_contact(sender):
                # Если есть, спрашиваем о желании открыть с ним чат и открываем при желании
                if self.messages.question(self, 'Новое сообщение',
                                          f'Получено новое сообщение от {sender}, '
                                          f'открыть чат с ним?', QMessageBox.Yes,
                                          QMessageBox.No) == QMessageBox.Yes:
                    self.current_chat = sender
                    self.set_active_user()
            else:
                # Раз нет, спрашиваем хотим ли добавить юзера в контакты.
                if self.messages.question(self, 'Новое сообщение',
                                          f'Получено новое сообщение от {sender}.\n '
                                          f'Данного пользователя нет в вашем контакт-листе.\n'
                                          f' Добавить в контакты и открыть чат с ним?',
                                          QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
                    self.transport.add_contact(sender)
                    self.current_chat = sender
                    self.update_contacts_list()
                    self.set_active_user()

    # Слот потери соединения
    # Выдаёт сообщение об ошибке и завершает работу приложения
    @pyqtSlot()
    def connection_lost(self):
        self.messages.warning(self, 'Сбой соединения', 'Потеряно соединение с сервером. ')
        self.close()

    @pyqtSlot()
    def sig_205(self):
        '''
        Слот выполняющий обновление баз данных по команде сервера.
        '''
        if self.current_chat and not self.database.check_user(
                self.current_chat):
            self.messages.warning(
                self,
                'Сочувствую',
                'К сожалению собеседник был удалён с сервера.')
            self.set_disabled_input()
            self.current_chat = None
        self.update_contacts_list()

    def make_connection(self, trans_obj):
        trans_obj.new_message.connect(self.message)
        trans_obj.connection_lost.connect(self.connection_lost)
        trans_obj.message_205.connect(self.sig_205)
        trans_obj.show_message.connect(self.show_message)


# class PopupWin(QDialog):
#     def __init__(self, msg):
#         super().__init__()
#         self.database = database
#
#         self.setFixedSize(350, 120)
#         self.setWindowTitle(f'Внимание')
#         self.setAttribute(Qt.WA_DeleteOnClose)
#         self.setModal(True)
#
#         self.selector_label = QLabel(msg)
#         self.selector_label.setFixedSize(330, 50)
#         self.selector_label.move(10, 0)
#
#         self.btn_ok = QPushButton('Ясно, понятно', self)
#         self.btn_ok.setFixedSize(150, 30)
#         self.btn_ok.move(100, 70)
#
#         self.show()

if __name__ == '__main__':
    app = QApplication(sys.argv)
    from client_db import ClientDB

    database = ClientDB('test1')
    from transport import ClientTransport

    transport = ClientTransport(database, 'test1', '127.0.0.1', 7777)
    window = ClientMainWindow(database, transport)
    sys.exit(app.exec_())
